@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 1993-2023 The Octave Project Developers
+// Copyright (C) 1993-2024 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -232,23 +232,15 @@ help_system::raw_help (const std::string& nm, bool& symbol_found) const
   return h;
 }
 
-std::string help_system::which (const std::string& name,
-                                std::string& type) const
+bool
+help_system::get_which_info_from_fcn (const std::string& name, const octave_value& ov_fcn, std::string& file, std::string& type) const
 {
-  std::string file;
-
-  if (name.empty ())
-    return file;
-
+  file = "";
   type = "";
 
-  symbol_table& symtab = m_interpreter.get_symbol_table ();
-
-  octave_value val = symtab.find_function (name);
-
-  if (val.is_defined ())
+  if (ov_fcn.is_function ())
     {
-      octave_function *fcn = val.function_value ();
+      octave_function *fcn = ov_fcn.function_value ();
 
       if (fcn)
         {
@@ -268,11 +260,10 @@ std::string help_system::which (const std::string& name,
             }
           else
             {
-
               file = fcn->fcn_file_name ();
 
               if (! file.empty ())
-                type = val.is_user_script () ? "script" : "function";
+                type = ov_fcn.is_user_script () ? "script" : "function";
               else
                 {
                   if (fcn->is_user_function ())
@@ -284,6 +275,8 @@ std::string help_system::which (const std::string& name,
                     }
                 }
             }
+
+          return true;
         }
       else
         {
@@ -292,35 +285,160 @@ std::string help_system::which (const std::string& name,
           load_path& lp = m_interpreter.get_load_path ();
 
           file = lp.find_fcn_file (name);
+
+          if (file.empty ())
+            return false;
         }
     }
 
-  if (file.empty ())
+  return false;
+}
+
+// FIXME: There is a lot of duplication between the following function
+// and help_system::raw_help_from_symbol_table.  Some refactoring would
+// probably be useful.
+
+std::string
+help_system::which (const std::string& name,
+                    std::string& type) const
+{
+  std::string file;
+
+  if (name.empty ())
+    return file;
+
+  type = "";
+
+  symbol_table& symtab = m_interpreter.get_symbol_table ();
+
+  size_t pos = name.find ('.');
+
+  if (pos == std::string::npos)
     {
-      // File query.
+      // Simple name.  If not found, continue looking for packages and
+      // classes.
 
-      load_path& lp = m_interpreter.get_load_path ();
+      octave_value ov_fcn = symtab.find_function (name);
 
-      // For compatibility: "file." queries "file".
-      if (name.size () > 1 && name[name.size () - 1] == '.')
-        file = lp.find_file (name.substr (0, name.size () - 1));
-      else
-        file = lp.find_file (name);
-
-      file = sys::env::make_absolute (file);
+      if (get_which_info_from_fcn (name, ov_fcn, file, type))
+        return file;
     }
 
+  // NAME contains '.' and must match the following pattern:
+  //
+  //   (package.)*(package|classname).(property|method)*
+  //
+  // Start by looking up the full name.  It could be either a package or
+  // a class and we are done.  Otherwise, strip the final component and
+  // lookup that name.  If it is a package, look for a function.  If it
+  // is a class, look for a property or method.
+
+  cdef_manager& cdm = m_interpreter.get_cdef_manager ();
+
+  // FIXME: In the following search we may load classes.  Is that really
+  // what we want, or should we just search the loadpath for
+  // +pkga/+pkgb/classname/file.m, etc. and attempt to extract help text
+  // without actually installing packages and classes into the fcn_info
+  // table?
+
+  // Is NAME a class?
+
+  cdef_class cls = cdm.find_class (name, false, true);
+
+  if (cls.ok ())
+    {
+      // FIXME: Return documentation for the class or the class
+      // constructor?
+
+      file = cls.file_name ();
+      type = "classdef class";
+
+      return file;
+    }
+
+  cdef_package pkg = cdm.find_package (name, false, true);
+
+  if (pkg.ok ())
+    {
+      // FIXME: How to get the fill name of a package?
+      file = pkg.get_name ();
+      type = "package";
+
+      return file;
+    }
+
+  // Strip final component (might be a property or method name).
+
+  pos = name.rfind ('.');
+  std::string prefix = name.substr (0, pos);
+  std::string nm = name.substr (pos+1);
+
+  // Is PREFIX the name of a class?
+
+  cls = cdm.find_class (prefix, false, true);
+
+  if (cls.ok ())
+    {
+      // FIXME: Should we only find public methods here?
+
+      octave_value ov_meth = cls.get_method (nm);
+
+      if (get_which_info_from_fcn (nm, ov_meth, file, type))
+        return file;
+
+      // FIXME: Should we only find public properties here?
+
+      cdef_property prop = cls.find_property (nm);
+
+      if (prop.ok ())
+        {
+          file = cls.file_name ();
+          type = "class property";
+
+          return file;
+        }
+    }
+
+  // Or is PREFIX the name of a package?
+
+  pkg = cdm.find_package (prefix, false, true);
+
+  if (pkg.ok ())
+    {
+      octave_value ov_fcn = pkg.find (nm);
+
+      if (get_which_info_from_fcn (nm, ov_fcn, file, type))
+        return file;
+    }
+
+  // File query.
+
+  load_path& lp = m_interpreter.get_load_path ();
+
+  // For compatibility: "file." queries "file".
+  if (name.size () > 1 && name[name.size () - 1] == '.')
+    file = lp.find_file (name.substr (0, name.size () - 1));
+  else
+    file = lp.find_file (name);
+
+  file = sys::env::make_absolute (file);
+
+  if (! file.empty ())
+    type = "file";
+
   return file;
 }
 
-std::string help_system::which (const std::string& name) const
+std::string
+help_system::which (const std::string& name) const
 {
   std::string type;
 
   return which (name, type);
 }
 
-string_vector help_system::make_name_list (void) const
+string_vector
+help_system::make_name_list () const
 {
   const static string_vector keywords
     = Fiskeyword ()(0).string_vector_value ();
@@ -383,8 +501,9 @@ string_vector help_system::make_name_list (void) const
   return list;
 }
 
-void help_system::get_help_text (const std::string& name, std::string& text,
-                                 std::string& format) const
+void
+help_system::get_help_text (const std::string& name, std::string& text,
+                            std::string& format) const
 {
   bool symbol_found = false;
   text = raw_help (name, symbol_found);
@@ -413,9 +532,10 @@ void help_system::get_help_text (const std::string& name, std::string& text,
     }
 }
 
-void help_system::get_help_text_from_file (const std::string& fname,
-    std::string& text,
-    std::string& format) const
+void
+help_system::get_help_text_from_file (const std::string& fname,
+                                      std::string& text,
+                                      std::string& format) const
 {
   bool symbol_found = false;
 
@@ -447,7 +567,8 @@ void help_system::get_help_text_from_file (const std::string& fname,
     }
 }
 
-std::string help_system::init_built_in_docstrings_file (void)
+std::string
+help_system::init_built_in_docstrings_file ()
 {
   std::string df = sys::env::getenv ("OCTAVE_BUILT_IN_DOCSTRINGS_FILE");
 
@@ -459,7 +580,8 @@ std::string help_system::init_built_in_docstrings_file (void)
   return df;
 }
 
-std::string help_system::init_doc_cache_file (void)
+std::string
+help_system::init_doc_cache_file ()
 {
   std::string def_file = config::prepend_octave_home (OCTAVE_DOC_CACHE_FILE);
 
@@ -468,7 +590,8 @@ std::string help_system::init_doc_cache_file (void)
   return (env_file.empty () ? def_file : env_file);
 }
 
-std::string help_system::init_info_file (void)
+std::string
+help_system::init_info_file ()
 {
   std::string std_info_file = config::prepend_octave_home (OCTAVE_INFOFILE);
 
@@ -477,7 +600,8 @@ std::string help_system::init_info_file (void)
   return (oct_info_file.empty () ? std_info_file : oct_info_file);
 }
 
-std::string help_system::init_info_program (void)
+std::string
+help_system::init_info_program ()
 {
   std::string info_prog = sys::env::getenv ("OCTAVE_INFO_PROGRAM");
 
@@ -487,7 +611,8 @@ std::string help_system::init_info_program (void)
   return info_prog;
 }
 
-std::string help_system::init_texi_macros_file (void)
+std::string
+help_system::init_texi_macros_file ()
 {
   std::string def_file
     = config::prepend_octave_home (OCTAVE_TEXI_MACROS_FILE);
@@ -499,7 +624,8 @@ std::string help_system::init_texi_macros_file (void)
 
 // Return a vector of all functions from this file,
 // for use in command line auto-completion.
-string_vector help_system::local_functions (void) const
+string_vector
+help_system::local_functions () const
 {
   string_vector retval;
 
@@ -534,56 +660,186 @@ string_vector help_system::local_functions (void) const
   return retval;
 }
 
-bool help_system::raw_help_from_symbol_table (const std::string& nm,
-    std::string& h, std::string& w,
-    bool& symbol_found) const
+static bool
+get_help_from_fcn (const std::string& fcn_nm, const octave_value& ov_fcn, std::string& help, std::string& what, bool& symbol_found)
 {
-  std::string meth_nm;
+  symbol_found = false;
 
+  help = "";
+  what = "";
+
+  if (ov_fcn.is_function ())
+    {
+      octave_function *fcn = ov_fcn.function_value ();
+
+      help = fcn->doc_string (fcn_nm);
+      what = fcn->fcn_file_name ();
+
+      if (what.empty ())
+        what = fcn->is_user_function () ? "command-line function" : "built-in function";
+
+      symbol_found = true;
+    }
+
+  return symbol_found;
+}
+
+// FIXME: There is a lot of duplication between the following function
+// and help_system::which.  Some refactoring would probably be useful.
+
+bool
+help_system::raw_help_from_symbol_table (const std::string& name, std::string& help, std::string& what, bool& symbol_found) const
+{
   symbol_table& symtab = m_interpreter.get_symbol_table ();
 
-  octave_value val = symtab.find_function (nm);
+  size_t pos = name.find ('.');
 
-  if (! val.is_defined ())
+  if (pos == std::string::npos)
     {
-      std::size_t pos = nm.rfind ('.');
+      // Simple name.  If not found, continue looking for packages and
+      // classes.
+
+      octave_value ov_fcn = symtab.find_function (name);
+
+      // FIXME: it seems like there is a lot of potential for confusion
+      // because is_function can also return true for
+      // octave_classdef_meta objects.
+
+      if (! ov_fcn.is_classdef_meta ()
+          && get_help_from_fcn (name, ov_fcn, help, what, symbol_found))
+        return true;
+    }
+
+  // If NAME does not contain '.', then it should be a package or a
+  // class name.
+  //
+  // If NAME contains '.' it should match the following pattern:
+  //
+  //   (package.)*(package|classname).(property|method)*
+  //
+  // Start by looking up the full name.  It could be either a package or
+  // a class and we are done.  Otherwise, strip the final component and
+  // lookup that name.  If it is a package, look for a function.  If it
+  // is a class, look for a property or method.
+
+  cdef_manager& cdm = m_interpreter.get_cdef_manager ();
+
+  // FIXME: In the following search we may load classes.  Is that really
+  // what we want, or should we just search the loadpath for
+  // +pkga/+pkgb/classname/file.m, etc. and attempt to extract help text
+  // without actually installing packages and classes into the fcn_info
+  // table?
+
+  // Is NAME a class?
+
+  cdef_class cls = cdm.find_class (name, false, true);
+
+  if (cls.ok ())
+    {
+      // Is the class documented?
+
+      help = cls.doc_string ();
+
+      if (! help.empty ())
+        {
+          what = "class";
+
+          symbol_found = true;
+          return true;
+        }
+
+      // Look for constructor.
+
+      pos = name.rfind ('.');
 
       if (pos != std::string::npos)
         {
-          meth_nm = nm.substr (pos+1);
+          std::string nm = name.substr (pos+1);
 
-          val = symtab.find_function (nm.substr (0, pos));
+          octave_value ov_meth = cls.get_method (nm);
+
+          if (get_help_from_fcn (nm, ov_meth, help, what, symbol_found))
+            {
+              what = "constructor";
+              return true;
+            }
+        }
+
+      // We found a class but no doc string for it or its constructor.
+      // Create a generic doc string.
+
+      help = name + " is an undocumented class";
+      what = "class";
+      symbol_found = true;
+      return true;
+    }
+
+  cdef_package pkg = cdm.find_package (name, false, true);
+
+  if (pkg.ok ())
+    {
+      help = "package " + name;
+      what = "package";
+
+      symbol_found = true;
+      return true;
+    }
+
+  // Strip final component (might be a property or method name).
+
+  pos = name.rfind ('.');
+  std::string prefix = name.substr (0, pos);
+  std::string nm = name.substr (pos+1);
+
+  // Is PREFIX the name of a class?
+
+  cls = cdm.find_class (prefix, false, true);
+
+  if (cls.ok ())
+    {
+      // FIXME: Should we only find public methods here?
+
+      octave_value ov_meth = cls.get_method (nm);
+
+      if (get_help_from_fcn (nm, ov_meth, help, what, symbol_found))
+        return true;
+
+      // FIXME: Should we only find public properties here?
+
+      cdef_property prop = cls.find_property (nm);
+
+      if (prop.ok ())
+        {
+          // FIXME: is it supposed to be possible to document
+          // properties?
+
+          help = prop.doc_string ();
+          what = "class property";
+
+          symbol_found = true;
+          return true;
         }
     }
 
-  if (val.is_defined ())
+  // Or is PREFIX the name of a package?
+
+  pkg = cdm.find_package (prefix, false, true);
+
+  if (pkg.ok ())
     {
-      octave_function *fcn = val.function_value ();
+      octave_value ov_fcn = pkg.find (nm);
 
-      if (fcn)
-        {
-          // FCN may actually be a classdef_meta object.
-
-          symbol_found = true;
-
-          h = fcn->doc_string (meth_nm);
-
-          w = fcn->fcn_file_name ();
-
-          if (w.empty ())
-            w = fcn->is_user_function () ? "command-line function"
-                : "built-in function";
-
-          return true;
-        }
+      if (get_help_from_fcn (nm, ov_fcn, help, what, symbol_found))
+        return true;
     }
 
   return false;
 }
 
-bool help_system::raw_help_from_file (const std::string& nm,
-                                      std::string& h, std::string& file,
-                                      bool& symbol_found) const
+bool
+help_system::raw_help_from_file (const std::string& nm,
+                                 std::string& h, std::string& file,
+                                 bool& symbol_found) const
 {
   bool retval = false;
 
@@ -725,7 +981,8 @@ help_system::raw_help_from_docstrings_file (const std::string& nm,
 
 // FIXME: It's not likely that this does the right thing now.
 
-string_vector make_name_list (void)
+string_vector
+make_name_list ()
 {
   help_system& help_sys = __get_help_system__ ();
 
