@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 1993-2023 The Octave Project Developers
+// Copyright (C) 1993-2024 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -43,6 +43,7 @@
 #include "fpucw-wrappers.h"
 #include "lo-blas-proto.h"
 #include "lo-error.h"
+#include "lo-sysdep.h"
 #include "oct-env.h"
 #include "quit.h"
 #include "str-vec.h"
@@ -91,7 +92,7 @@ bool quit_allowed = true;
 bool octave_interpreter_ready = false;
 
 // TRUE means we've processed all the init code and we are good to go.
-bool octave_initialized = false;
+std::atomic<bool> octave_initialized{false};
 
 OCTAVE_BEGIN_NAMESPACE(octave)
 
@@ -148,10 +149,9 @@ DEFMETHOD (quit, interp, args, ,
 @deftypefnx {} {} quit ("force")
 @deftypefnx {} {} quit (@var{status})
 @deftypefnx {} {} quit (@var{status}, "force")
+@deftypefnx {} {} exit (@dots{})
 Quit the current Octave session.
 
-The @code{exit} function is an alias for @code{quit}.
-
 If the optional integer value @var{status} is supplied, pass that value to
 the operating system as Octave's exit status.  The default value is zero.
 
@@ -172,6 +172,9 @@ process.
 
 If the optional argument @qcode{"force"} is provided, no confirmation is
 requested, and the execution of the @file{finish.m} file is skipped.
+
+Programming Note: @code{exit} is an alias for @code{quit} and can be used
+interchangeably.
 @seealso{atexit}
 @end deftypefn */)
 {
@@ -314,17 +317,19 @@ Return true if Octave was invoked with the @w{@env{--traditional}} option.
   return ovl (interp.traditional ());
 }
 
-temporary_file_list::~temporary_file_list (void)
+temporary_file_list::~temporary_file_list ()
 {
   cleanup ();
 }
 
-void temporary_file_list::insert (const std::string& file)
+void
+temporary_file_list::insert (const std::string& file)
 {
   m_files.insert (file);
 }
 
-void temporary_file_list::cleanup (void)
+void
+temporary_file_list::cleanup ()
 {
   while (! m_files.empty ())
     {
@@ -339,56 +344,27 @@ void temporary_file_list::cleanup (void)
 // The time we last time we changed directories.
 sys::time Vlast_chdir_time = 0.0;
 
-// Execute commands from a file and catch potential exceptions in a consistent
-// way.  This function should be called anywhere we might parse and execute
-// commands from a file before we have entered the main loop in
-// toplev.cc.
-
-static int safe_source_file (const std::string& file_name,
-                             const std::string& context = "",
-                             bool verbose = false,
-                             bool require_file = true)
+static void
+initialize_version_info ()
 {
-  interpreter& interp = __get_interpreter__ ();
+  octave_value_list args (4);
 
-  try
-    {
-      source_file (file_name, context, verbose, require_file);
-    }
-  catch (const interrupt_exception&)
-    {
-      interp.recover_from_exception ();
-
-      return 1;
-    }
-  catch (const execution_exception& ee)
-    {
-      interp.handle_exception (ee);
-
-      return 1;
-    }
-
-  return 0;
-}
-
-static void initialize_version_info (void)
-{
-  octave_value_list args;
-
-  args(3) = OCTAVE_RELEASE_DATE;
-  args(2) = config::release ();
-  args(1) = OCTAVE_VERSION;
   args(0) = "GNU Octave";
+  args(1) = OCTAVE_VERSION;
+  args(2) = config::release ();
+  args(3) = OCTAVE_RELEASE_DATE;
 
-  F__version_info__ (args, 0);
+  F__version_info__ (args);
 }
 
-static void xerbla_abort (void)
+static void
+xerbla_abort ()
 {
   error ("Fortran procedure terminated by call to XERBLA");
 }
 
-static void initialize_xerbla_error_handler (void)
+static void
+initialize_xerbla_error_handler ()
 {
   // The idea here is to force xerbla to be referenced so that we will
   // link to our own version instead of the one provided by the BLAS
@@ -400,7 +376,7 @@ static void initialize_xerbla_error_handler (void)
   if (numeric_limits<double>::NaN () == -1)
     F77_FUNC (xerbla, XERBLA) ("octave", 13 F77_CHAR_ARG_LEN (6));
 
-  typedef void (*xerbla_handler_ptr) (void);
+  typedef void (*xerbla_handler_ptr) ();
 
   typedef void (*octave_set_xerbla_handler_ptr) (xerbla_handler_ptr);
 
@@ -439,7 +415,8 @@ lo_error_with_id_handler (const char *id, const char *fmt, ...)
   throw execution_exception ();
 }
 
-static void initialize_error_handlers (void)
+static void
+initialize_error_handlers ()
 {
   set_liboctave_error_handler (lo_error_handler);
   set_liboctave_error_with_id_handler (lo_error_with_id_handler);
@@ -473,7 +450,7 @@ interpreter::interpreter (application *app_context)
     m_child_list (),
     m_url_handle_manager (),
     m_cdef_manager (*this),
-    m_gtk_manager (),
+    m_gtk_manager (*this),
     m_event_manager (*this),
     m_gh_manager (nullptr),
     m_interactive (false),
@@ -495,11 +472,11 @@ interpreter::interpreter (application *app_context)
   //
   //   only one Octave interpreter may be active in any given thread
 
-  if (m_instance)
+  if (s_instance)
     throw std::runtime_error
     ("only one Octave interpreter may be active");
 
-  m_instance = this;
+  s_instance = this;
 
 #if defined (OCTAVE_HAVE_WINDOWS_UTF8_LOCALE)
   // Force a UTF-8 locale on Windows if possible
@@ -607,23 +584,23 @@ interpreter::interpreter (application *app_context)
 
       std::string docstrings_file = options.docstrings_file ();
       if (! docstrings_file.empty ())
-        Fbuilt_in_docstrings_file (*this, octave_value (docstrings_file));
+        Fbuilt_in_docstrings_file (*this, ovl (docstrings_file));
 
       std::string doc_cache_file = options.doc_cache_file ();
       if (! doc_cache_file.empty ())
-        Fdoc_cache_file (*this, octave_value (doc_cache_file));
+        Fdoc_cache_file (*this, ovl (doc_cache_file));
 
       std::string info_file = options.info_file ();
       if (! info_file.empty ())
-        Finfo_file (*this, octave_value (info_file));
+        Finfo_file (*this, ovl (info_file));
 
       std::string info_program = options.info_program ();
       if (! info_program.empty ())
-        Finfo_program (*this, octave_value (info_program));
+        Finfo_program (*this, ovl (info_program));
 
       std::string texi_macros_file = options.texi_macros_file ();
       if (! texi_macros_file.empty ())
-        Ftexi_macros_file (*this, octave_value (texi_macros_file));
+        Ftexi_macros_file (*this, ovl (texi_macros_file));
     }
 
   // FIXME: we defer creation of the gh_manager object because it
@@ -655,9 +632,9 @@ interpreter::interpreter (application *app_context)
   octave_interpreter_ready = true;
 }
 
-OCTAVE_THREAD_LOCAL interpreter *interpreter::m_instance = nullptr;
+OCTAVE_THREAD_LOCAL interpreter *interpreter::s_instance = nullptr;
 
-interpreter::~interpreter (void)
+interpreter::~interpreter ()
 {
   if (! m_app_context)
     shutdown ();
@@ -665,14 +642,16 @@ interpreter::~interpreter (void)
   delete m_gh_manager;
 }
 
-void interpreter::intern_nargin (octave_idx_type nargs)
+void
+interpreter::intern_nargin (octave_idx_type nargs)
 {
   m_evaluator.set_auto_fcn_var (stack_frame::NARGIN, nargs);
 }
 
 // Read the history file unless a command-line option inhibits that.
 
-void interpreter::initialize_history (bool read_history_file)
+void
+interpreter::initialize_history (bool read_history_file)
 {
   if (! m_history_initialized)
     {
@@ -700,7 +679,8 @@ void interpreter::initialize_history (bool read_history_file)
 // Set the initial path to the system default unless command-line
 // option says to leave it empty.
 
-void interpreter::initialize_load_path (bool set_initial_path)
+void
+interpreter::initialize_load_path (bool set_initial_path)
 {
   if (! m_load_path_initialized)
     {
@@ -736,7 +716,8 @@ void interpreter::initialize_load_path (bool set_initial_path)
 
 // This may be called separately from execute
 
-void interpreter::initialize (void)
+void
+interpreter::initialize ()
 {
   if (m_initialized)
     return;
@@ -789,7 +770,8 @@ void interpreter::initialize (void)
 // Note: this function is currently only used with the new
 // experimental terminal widget.
 
-void interpreter::get_line_and_eval (void)
+void
+interpreter::get_line_and_eval ()
 {
   m_evaluator.get_line_and_eval ();
 }
@@ -804,11 +786,9 @@ public:
   cli_input_reader (interpreter& interp)
     : m_interpreter (interp), m_thread () { }
 
-  cli_input_reader (const cli_input_reader&) = delete;
+  OCTAVE_DISABLE_CONSTRUCT_COPY_MOVE (cli_input_reader)
 
-  cli_input_reader& operator = (const cli_input_reader&) = delete;
-
-  ~cli_input_reader (void)
+  ~cli_input_reader ()
   {
     // FIXME: Would it be better to ensure that
     // interpreter::get_line_and_eval exits and then call
@@ -817,7 +797,7 @@ public:
     m_thread.detach ();
   }
 
-  void start (void)
+  void start ()
   {
     m_thread = std::thread (&interpreter::get_line_and_eval, &m_interpreter);
   }
@@ -829,8 +809,9 @@ private:
   std::thread m_thread;
 };
 
-void interpreter::parse_and_execute (const std::string& input,
-                                     bool& incomplete_parse)
+void
+interpreter::parse_and_execute (const std::string& input,
+                                bool& incomplete_parse)
 {
   m_evaluator.parse_and_execute (input, incomplete_parse);
 }
@@ -838,7 +819,8 @@ void interpreter::parse_and_execute (const std::string& input,
 // FIXME: this function is intended to be executed only once.  Should
 // we enforce that restriction?
 
-int interpreter::execute (void)
+int
+interpreter::execute ()
 {
   int exit_status = 0;
 
@@ -965,7 +947,8 @@ int interpreter::execute (void)
     }                                                                   \
   while (0)
 
-void interpreter::shutdown (void)
+void
+interpreter::shutdown ()
 {
   // Attempt to prevent more than one call to shutdown.
 
@@ -1075,7 +1058,8 @@ void interpreter::shutdown (void)
   // OCTAVE_SAFE_CALL (singleton_cleanup_list::cleanup, ());
 }
 
-void interpreter::execute_atexit_fcns (void)
+void
+interpreter::execute_atexit_fcns ()
 {
   // Prevent atexit functions from adding new functions to the list.
   m_executing_atexit = true;
@@ -1092,7 +1076,8 @@ void interpreter::execute_atexit_fcns (void)
     }
 }
 
-void interpreter::display_startup_message (void) const
+void
+interpreter::display_startup_message () const
 {
   bool inhibit_startup_message = false;
 
@@ -1107,11 +1092,12 @@ void interpreter::display_startup_message (void) const
     std::cout << octave_startup_message () << "\n" << std::endl;
 }
 
-// Initialize by reading startup files.  Return non-zero if an exception
+// Initialize by reading startup files.  Return nonzero if an exception
 // occurs when reading any of them, but don't exit early because of an
 // exception.
 
-int interpreter::execute_startup_files (void)
+int
+interpreter::execute_startup_files ()
 {
   bool read_site_files = m_read_site_files;
   bool read_init_files = m_read_init_files;
@@ -1227,16 +1213,14 @@ int interpreter::execute_startup_files (void)
 
           // Names alone are not enough.
 
-          sys::file_stat fs_home_rc (home_rc);
-
-          if (fs_home_rc)
+          if (sys::file_exists (home_rc))
             {
               // We want to check for curr_dir after executing home_rc
               // because doing that may change the working directory.
 
               local_rc = sys::env::make_absolute (initfile);
 
-              home_rc_already_executed = same_file (home_rc, local_rc);
+              home_rc_already_executed = sys::same_file (home_rc, local_rc);
             }
         }
 
@@ -1261,7 +1245,8 @@ int interpreter::execute_startup_files (void)
 
 // Execute any code specified with --eval 'CODE'
 
-int interpreter::execute_eval_option_code (void)
+int
+interpreter::execute_eval_option_code ()
 {
   if (! m_app_context)
     return 0;
@@ -1294,7 +1279,8 @@ int interpreter::execute_eval_option_code (void)
   return parse_status;
 }
 
-int interpreter::execute_command_line_file (void)
+int
+interpreter::execute_command_line_file ()
 {
   if (! m_app_context)
     return 0;
@@ -1343,41 +1329,46 @@ int interpreter::execute_command_line_file (void)
   return safe_source_file (fname, context, verbose, require_file);
 }
 
-int interpreter::main_loop (void)
+int
+interpreter::main_loop ()
 {
   command_editor::add_event_hook (release_unreferenced_dynamic_libraries);
 
   return m_evaluator.repl ();
 }
 
-int interpreter::server_loop (void)
+int
+interpreter::server_loop ()
 {
   return m_evaluator.server_loop ();
 }
 
-tree_evaluator& interpreter::get_evaluator (void)
+tree_evaluator&
+interpreter::get_evaluator ()
 {
   return m_evaluator;
 }
 
-stream_list& interpreter::get_stream_list (void)
+stream_list&
+interpreter::get_stream_list ()
 {
   return m_stream_list;
 }
 
-url_handle_manager& interpreter::get_url_handle_manager (void)
+url_handle_manager&
+interpreter::get_url_handle_manager ()
 {
   return m_url_handle_manager;
 }
 
 symbol_scope
-interpreter::get_top_scope (void) const
+interpreter::get_top_scope () const
 {
   return m_evaluator.get_top_scope ();
 }
 
 symbol_scope
-interpreter::get_current_scope (void) const
+interpreter::get_current_scope () const
 {
   return m_evaluator.get_current_scope ();
 }
@@ -1393,12 +1384,14 @@ interpreter::require_current_scope (const std::string& who) const
   return scope;
 }
 
-profiler& interpreter::get_profiler (void)
+profiler&
+interpreter::get_profiler ()
 {
   return m_evaluator.get_profiler ();
 }
 
-int interpreter::chdir (const std::string& dir)
+int
+interpreter::chdir (const std::string& dir)
 {
   std::string xdir = sys::file_ops::tilde_expand (dir);
 
@@ -1420,22 +1413,26 @@ int interpreter::chdir (const std::string& dir)
   return cd_ok;
 }
 
-void interpreter::mlock (bool skip_first) const
+void
+interpreter::mlock (bool skip_first) const
 {
   m_evaluator.mlock (skip_first);
 }
 
-void interpreter::munlock (bool skip_first) const
+void
+interpreter::munlock (bool skip_first) const
 {
   m_evaluator.munlock (skip_first);
 }
 
-bool interpreter::mislocked (bool skip_first) const
+bool
+interpreter::mislocked (bool skip_first) const
 {
   return m_evaluator.mislocked (skip_first);
 }
 
-void interpreter::munlock (const char *nm)
+void
+interpreter::munlock (const char *nm)
 {
   if (! nm)
     error ("munlock: invalid value for NAME");
@@ -1443,7 +1440,8 @@ void interpreter::munlock (const char *nm)
   munlock (std::string (nm));
 }
 
-void interpreter::munlock (const std::string& nm)
+void
+interpreter::munlock (const std::string& nm)
 {
   octave_value val = m_symbol_table.find_function (nm);
 
@@ -1456,7 +1454,8 @@ void interpreter::munlock (const std::string& nm)
     }
 }
 
-bool interpreter::mislocked (const char *nm)
+bool
+interpreter::mislocked (const char *nm)
 {
   if (! nm)
     error ("mislocked: invalid value for NAME");
@@ -1464,7 +1463,8 @@ bool interpreter::mislocked (const char *nm)
   return mislocked (std::string (nm));
 }
 
-bool interpreter::mislocked (const std::string& nm)
+bool
+interpreter::mislocked (const std::string& nm)
 {
   bool retval = false;
 
@@ -1481,55 +1481,63 @@ bool interpreter::mislocked (const std::string& nm)
   return retval;
 }
 
-std::string interpreter::mfilename (const std::string& opt) const
+std::string
+interpreter::mfilename (const std::string& opt) const
 {
   return m_evaluator.mfilename (opt);
 }
 
-octave_value_list interpreter::eval_string (const std::string& eval_str,
-    bool silent, int& parse_status,
-    int nargout)
+octave_value_list
+interpreter::eval_string (const std::string& eval_str,
+                          bool silent, int& parse_status,
+                          int nargout)
 {
   return m_evaluator.eval_string (eval_str, silent, parse_status, nargout);
 }
 
-octave_value interpreter::eval_string (const std::string& eval_str,
-                                       bool silent, int& parse_status)
+octave_value
+interpreter::eval_string (const std::string& eval_str,
+                          bool silent, int& parse_status)
 {
   return m_evaluator.eval_string (eval_str, silent, parse_status);
 }
 
-octave_value_list interpreter::eval_string (const octave_value& arg,
-    bool silent, int& parse_status,
-    int nargout)
+octave_value_list
+interpreter::eval_string (const octave_value& arg,
+                          bool silent, int& parse_status,
+                          int nargout)
 {
   return m_evaluator.eval_string (arg, silent, parse_status, nargout);
 }
 
-octave_value_list interpreter::eval (const std::string& try_code,
-                                     int nargout)
+octave_value_list
+interpreter::eval (const std::string& try_code,
+                   int nargout)
 {
   return m_evaluator.eval (try_code, nargout);
 }
 
-octave_value_list interpreter::eval (const std::string& try_code,
-                                     const std::string& catch_code,
-                                     int nargout)
+octave_value_list
+interpreter::eval (const std::string& try_code,
+                   const std::string& catch_code,
+                   int nargout)
 {
   return m_evaluator.eval (try_code, catch_code, nargout);
 }
 
-octave_value_list interpreter::evalin (const std::string& context,
-                                       const std::string& try_code,
-                                       int nargout)
+octave_value_list
+interpreter::evalin (const std::string& context,
+                     const std::string& try_code,
+                     int nargout)
 {
   return m_evaluator.evalin (context, try_code, nargout);
 }
 
-octave_value_list interpreter::evalin (const std::string& context,
-                                       const std::string& try_code,
-                                       const std::string& catch_code,
-                                       int nargout)
+octave_value_list
+interpreter::evalin (const std::string& context,
+                     const std::string& try_code,
+                     const std::string& catch_code,
+                     int nargout)
 {
   return m_evaluator.evalin (context, try_code, catch_code, nargout);
 }
@@ -1543,16 +1551,18 @@ octave_value_list interpreter::evalin (const std::string& context,
 //! @return A list of output values.  The length of the list is not
 //!         necessarily the same as @c nargout.
 
-octave_value_list interpreter::feval (const char *name,
-                                      const octave_value_list& args,
-                                      int nargout)
+octave_value_list
+interpreter::feval (const char *name,
+                    const octave_value_list& args,
+                    int nargout)
 {
   return feval (std::string (name), args, nargout);
 }
 
-octave_value_list interpreter::feval (const std::string& name,
-                                      const octave_value_list& args,
-                                      int nargout)
+octave_value_list
+interpreter::feval (const std::string& name,
+                    const octave_value_list& args,
+                    int nargout)
 {
   octave_value fcn = m_symbol_table.find_function (name, args);
 
@@ -1564,9 +1574,10 @@ octave_value_list interpreter::feval (const std::string& name,
   return of->call (m_evaluator, nargout, args);
 }
 
-octave_value_list interpreter::feval (octave_function *fcn,
-                                      const octave_value_list& args,
-                                      int nargout)
+octave_value_list
+interpreter::feval (octave_function *fcn,
+                    const octave_value_list& args,
+                    int nargout)
 {
   if (fcn)
     return fcn->call (m_evaluator, nargout, args);
@@ -1574,9 +1585,10 @@ octave_value_list interpreter::feval (octave_function *fcn,
   return octave_value_list ();
 }
 
-octave_value_list interpreter::feval (const octave_value& val,
-                                      const octave_value_list& args,
-                                      int nargout)
+octave_value_list
+interpreter::feval (const octave_value& val,
+                    const octave_value_list& args,
+                    int nargout)
 {
   // FIXME: do we really want to silently return an empty ovl if
   // the function object is undefined?  It's essentially what the
@@ -1629,8 +1641,9 @@ octave_value_list interpreter::feval (const octave_value& val,
 //! @return A list of output values.  The length of the list is not
 //!         necessarily the same as @c nargout.
 
-octave_value_list interpreter::feval (const octave_value_list& args,
-                                      int nargout)
+octave_value_list
+interpreter::feval (const octave_value_list& args,
+                    int nargout)
 {
   if (args.length () == 0)
     error ("feval: first argument must be a string, inline function, or a function handle");
@@ -1642,195 +1655,232 @@ octave_value_list interpreter::feval (const octave_value_list& args,
   return feval (f_arg, tmp_args, nargout);
 }
 
-octave_value interpreter::make_function_handle (const std::string& name)
+octave_value
+interpreter::make_function_handle (const std::string& name)
 {
   return m_evaluator.make_fcn_handle (name);
 }
 
-void interpreter::install_variable (const std::string& name,
-                                    const octave_value& value, bool global)
+void
+interpreter::install_variable (const std::string& name,
+                               const octave_value& value, bool global)
 {
   m_evaluator.install_variable (name, value, global);
 }
 
-octave_value interpreter::global_varval (const std::string& name) const
+octave_value
+interpreter::global_varval (const std::string& name) const
 {
   return m_evaluator.global_varval (name);
 }
 
-void interpreter::global_assign (const std::string& name,
-                                 const octave_value& val)
+void
+interpreter::global_assign (const std::string& name,
+                            const octave_value& val)
 {
   m_evaluator.global_assign (name, val);
 }
 
-octave_value interpreter::top_level_varval (const std::string& name) const
+octave_value
+interpreter::top_level_varval (const std::string& name) const
 {
   return m_evaluator.top_level_varval (name);
 }
 
-void interpreter::top_level_assign (const std::string& name,
-                                    const octave_value& val)
+void
+interpreter::top_level_assign (const std::string& name,
+                               const octave_value& val)
 {
   m_evaluator.top_level_assign (name, val);
 }
 
-bool interpreter::is_variable (const std::string& name) const
+bool
+interpreter::is_variable (const std::string& name) const
 {
   return m_evaluator.is_variable (name);
 }
 
-bool interpreter::is_local_variable (const std::string& name) const
+bool
+interpreter::is_local_variable (const std::string& name) const
 {
   return m_evaluator.is_local_variable (name);
 }
 
-octave_value interpreter::varval (const std::string& name) const
+octave_value
+interpreter::varval (const std::string& name) const
 {
   return m_evaluator.varval (name);
 }
 
-void interpreter::assign (const std::string& name,
-                          const octave_value& val)
+void
+interpreter::assign (const std::string& name,
+                     const octave_value& val)
 {
   m_evaluator.assign (name, val);
 }
 
-void interpreter::assignin (const std::string& context,
-                            const std::string& name,
-                            const octave_value& val)
+void
+interpreter::assignin (const std::string& context,
+                       const std::string& name,
+                       const octave_value& val)
 {
   m_evaluator.assignin (context, name, val);
 }
 
-void interpreter::source_file (const std::string& file_name,
-                               const std::string& context, bool verbose,
-                               bool require_file)
+void
+interpreter::source_file (const std::string& file_name,
+                          const std::string& context, bool verbose,
+                          bool require_file)
 {
   m_evaluator.source_file (file_name, context, verbose, require_file);
 }
 
-bool interpreter::at_top_level (void) const
+bool
+interpreter::at_top_level () const
 {
   return m_evaluator.at_top_level ();
 }
 
-bool interpreter::isglobal (const std::string& name) const
+bool
+interpreter::isglobal (const std::string& name) const
 {
   return m_evaluator.is_global (name);
 }
 
-octave_value interpreter::find (const std::string& name)
+octave_value
+interpreter::find (const std::string& name)
 {
   return m_evaluator.find (name);
 }
 
-void interpreter::clear_all (bool force)
+void
+interpreter::clear_all (bool force)
 {
   m_evaluator.clear_all (force);
 }
 
-void interpreter::clear_objects (void)
+void
+interpreter::clear_objects ()
 {
   m_evaluator.clear_objects ();
 }
 
-void interpreter::clear_variable (const std::string& name)
+void
+interpreter::clear_variable (const std::string& name)
 {
   m_evaluator.clear_variable (name);
 }
 
-void interpreter::clear_variable_pattern (const std::string& pattern)
+void
+interpreter::clear_variable_pattern (const std::string& pattern)
 {
   m_evaluator.clear_variable_pattern (pattern);
 }
 
-void interpreter::clear_variable_regexp (const std::string& pattern)
+void
+interpreter::clear_variable_regexp (const std::string& pattern)
 {
   m_evaluator.clear_variable_regexp (pattern);
 }
 
-void interpreter::clear_variables (void)
+void
+interpreter::clear_variables ()
 {
   m_evaluator.clear_variables ();
 }
 
-void interpreter::clear_global_variable (const std::string& name)
+void
+interpreter::clear_global_variable (const std::string& name)
 {
   m_evaluator.clear_global_variable (name);
 }
 
-void interpreter::clear_global_variable_pattern (const std::string& pattern)
+void
+interpreter::clear_global_variable_pattern (const std::string& pattern)
 {
   m_evaluator.clear_global_variable_pattern (pattern);
 }
 
-void interpreter::clear_global_variable_regexp (const std::string& pattern)
+void
+interpreter::clear_global_variable_regexp (const std::string& pattern)
 {
   m_evaluator.clear_global_variable_regexp (pattern);
 }
 
-void interpreter::clear_global_variables (void)
+void
+interpreter::clear_global_variables ()
 {
   m_evaluator.clear_global_variables ();
 }
 
-void interpreter::clear_functions (bool force)
+void
+interpreter::clear_functions (bool force)
 {
   m_symbol_table.clear_functions (force);
 }
 
-void interpreter::clear_function (const std::string& name)
+void
+interpreter::clear_function (const std::string& name)
 {
   m_symbol_table.clear_function (name);
 }
 
-void interpreter::clear_symbol (const std::string& name)
+void
+interpreter::clear_symbol (const std::string& name)
 {
   m_evaluator.clear_symbol (name);
 }
 
-void interpreter::clear_function_pattern (const std::string& pat)
+void
+interpreter::clear_function_pattern (const std::string& pat)
 {
   m_symbol_table.clear_function_pattern (pat);
 }
 
-void interpreter::clear_function_regexp (const std::string& pat)
+void
+interpreter::clear_function_regexp (const std::string& pat)
 {
   m_symbol_table.clear_function_regexp (pat);
 }
 
-void interpreter::clear_symbol_pattern (const std::string& pat)
+void
+interpreter::clear_symbol_pattern (const std::string& pat)
 {
   return m_evaluator.clear_symbol_pattern (pat);
 }
 
-void interpreter::clear_symbol_regexp (const std::string& pat)
+void
+interpreter::clear_symbol_regexp (const std::string& pat)
 {
   return m_evaluator.clear_symbol_regexp (pat);
 }
 
-std::list<std::string> interpreter::global_variable_names (void)
+std::list<std::string>
+interpreter::global_variable_names ()
 {
   return m_evaluator.global_variable_names ();
 }
 
-std::list<std::string> interpreter::top_level_variable_names (void)
+std::list<std::string>
+interpreter::top_level_variable_names ()
 {
   return m_evaluator.top_level_variable_names ();
 }
 
-std::list<std::string> interpreter::variable_names (void)
+std::list<std::string>
+interpreter::variable_names ()
 {
   return m_evaluator.variable_names ();
 }
 
-std::list<std::string> interpreter::user_function_names (void)
+std::list<std::string>
+interpreter::user_function_names ()
 {
   return m_symbol_table.user_function_names ();
 }
 
-std::list<std::string> interpreter::autoloaded_functions (void) const
+std::list<std::string>
+interpreter::autoloaded_functions () const
 {
   return m_evaluator.autoloaded_functions ();
 }
@@ -1838,7 +1888,8 @@ std::list<std::string> interpreter::autoloaded_functions (void) const
 // May be used to send an interrupt signal to the the interpreter from
 // another thread (for example, the GUI).
 
-void interpreter::interrupt (void)
+void
+interpreter::interrupt ()
 {
   static int sigint = 0;
   static bool first = true;
@@ -1866,7 +1917,8 @@ void interpreter::interrupt (void)
   octave_kill_wrapper (pid, sigint);
 }
 
-void interpreter::pause (void)
+void
+interpreter::pause ()
 {
   // FIXME: To be reliable, these tree_evaluator functions must be
   // made thread safe.
@@ -1875,7 +1927,8 @@ void interpreter::pause (void)
   m_evaluator.reset_debug_state ();
 }
 
-void interpreter::stop (void)
+void
+interpreter::stop ()
 {
   // FIXME: To be reliable, these tree_evaluator functions must be
   // made thread safe.
@@ -1886,7 +1939,8 @@ void interpreter::stop (void)
     interrupt ();
 }
 
-void interpreter::resume (void)
+void
+interpreter::resume ()
 {
   // FIXME: To be reliable, these tree_evaluator functions must be
   // made thread safe.
@@ -1898,9 +1952,82 @@ void interpreter::resume (void)
     m_evaluator.dbcont ();
 }
 
+octave_value
+interpreter::PS1 (const octave_value_list& args, int nargout)
+{
+  return m_input_system.PS1 (args, nargout);
+}
+
+std::string
+interpreter::PS1 () const
+{
+  return m_input_system.PS1 ();
+}
+
+std::string
+interpreter::PS1 (const std::string& s)
+{
+  return m_input_system.PS1 (s);
+}
+
+void
+interpreter::set_PS1 (const std::string& s)
+{
+  m_input_system.set_PS1 (s);
+}
+
+octave_value
+interpreter::PS2 (const octave_value_list& args, int nargout)
+{
+  return m_input_system.PS2 (args, nargout);
+}
+
+std::string
+interpreter::PS2 () const
+{
+  return m_input_system.PS2 ();
+}
+
+std::string
+interpreter::PS2 (const std::string& s)
+{
+  return m_input_system.PS2 (s);
+}
+
+void
+interpreter::set_PS2 (const std::string& s)
+{
+  m_input_system.set_PS2 (s);
+}
+
+octave_value
+interpreter::PS4 (const octave_value_list& args, int nargout)
+{
+  return m_evaluator.PS4 (args, nargout);
+}
+
+std::string
+interpreter::PS4 () const
+{
+  return m_evaluator.PS4 ();
+}
+
+std::string
+interpreter::PS4 (const std::string& s)
+{
+  return m_evaluator.PS4 (s);
+}
+
+void
+interpreter::set_PS4 (const std::string& s)
+{
+  m_evaluator.set_PS4 (s);
+}
+
 // Provided for convenience.  Will be removed once we eliminate the
 // old terminal widget.
-bool interpreter::experimental_terminal_widget (void) const
+bool
+interpreter::experimental_terminal_widget () const
 {
   if (! m_app_context)
     return false;
@@ -1911,27 +2038,32 @@ bool interpreter::experimental_terminal_widget (void) const
   return options.experimental_terminal_widget ();
 }
 
-void interpreter::add_debug_watch_expression (const std::string& expr)
+void
+interpreter::add_debug_watch_expression (const std::string& expr)
 {
   m_evaluator.add_debug_watch_expression (expr);
 }
 
-void interpreter::remove_debug_watch_expression (const std::string& expr)
+void
+interpreter::remove_debug_watch_expression (const std::string& expr)
 {
   m_evaluator.remove_debug_watch_expression (expr);
 }
 
-void interpreter::clear_debug_watch_expressions (void)
+void
+interpreter::clear_debug_watch_expressions ()
 {
   m_evaluator.clear_debug_watch_expressions ();
 }
 
-std::set<std::string> interpreter::debug_watch_expressions (void) const
+std::set<std::string>
+interpreter::debug_watch_expressions () const
 {
   return m_evaluator.debug_watch_expressions ();
 }
 
-void interpreter::handle_exception (const execution_exception& ee)
+void
+interpreter::handle_exception (const execution_exception& ee)
 {
   m_error_system.save_exception (ee);
 
@@ -1943,29 +2075,33 @@ void interpreter::handle_exception (const execution_exception& ee)
   recover_from_exception ();
 }
 
-void interpreter::recover_from_exception (void)
+void
+interpreter::recover_from_exception ()
 {
   if (octave_interrupt_state)
     m_event_manager.interpreter_interrupted ();
 
   can_interrupt = true;
   octave_interrupt_state = 0;
-  octave_signal_caught = 0;
+  octave_signal_caught = false;
   octave_restore_signal_mask ();
   catch_interrupts ();
 }
 
-void interpreter::mark_for_deletion (const std::string& file)
+void
+interpreter::mark_for_deletion (const std::string& file)
 {
   m_tmp_files.insert (file);
 }
 
-void interpreter::cleanup_tmp_files (void)
+void
+interpreter::cleanup_tmp_files ()
 {
   m_tmp_files.cleanup ();
 }
 
-void interpreter::quit (int exit_status, bool force, bool confirm)
+void
+interpreter::quit (int exit_status, bool force, bool confirm)
 {
   if (! force)
     {
@@ -2007,7 +2143,8 @@ void interpreter::quit (int exit_status, bool force, bool confirm)
   throw exit_exception (exit_status);
 }
 
-void interpreter::add_atexit_fcn (const std::string& fname)
+void
+interpreter::add_atexit_fcn (const std::string& fname)
 {
   if (m_executing_atexit)
     return;
@@ -2015,7 +2152,8 @@ void interpreter::add_atexit_fcn (const std::string& fname)
   m_atexit_fcns.push_front (fname);
 }
 
-bool interpreter::remove_atexit_fcn (const std::string& fname)
+bool
+interpreter::remove_atexit_fcn (const std::string& fname)
 {
   bool found = false;
 
@@ -2035,12 +2173,12 @@ bool interpreter::remove_atexit_fcn (const std::string& fname)
 
 // What internal options get configured by --traditional.
 
-void interpreter::maximum_braindamage (void)
+void
+interpreter::maximum_braindamage ()
 {
-  m_input_system.PS1 (">> ");
-  m_input_system.PS2 ("");
-
-  m_evaluator.PS4 ("");
+  PS1 (">> ");
+  PS2 ("");
+  PS4 ("");
 
   m_load_save_system.crash_dumps_octave_core (false);
   m_load_save_system.save_default_options ("-mat-binary");
@@ -2049,14 +2187,14 @@ void interpreter::maximum_braindamage (void)
 
   m_error_system.beep_on_error (true);
 
-  Fconfirm_recursive_rmdir (octave_value (false));
-  Foptimize_diagonal_matrix (octave_value (false));
-  Foptimize_permutation_matrix (octave_value (false));
-  Foptimize_range (octave_value (false));
-  Ffixed_point_format (octave_value (true));
-  Fprint_empty_dimensions (octave_value (false));
-  Fprint_struct_array_contents (octave_value (true));
-  Fstruct_levels_to_print (octave_value (0));
+  Fconfirm_recursive_rmdir (ovl (false));
+  Foptimize_diagonal_matrix (ovl (false));
+  Foptimize_permutation_matrix (ovl (false));
+  Foptimize_range (ovl (false));
+  Ffixed_point_format (ovl (true));
+  Fprint_empty_dimensions (ovl (false));
+  Fprint_struct_array_contents (ovl (true));
+  Fstruct_levels_to_print (ovl (0));
 
   m_error_system.disable_warning ("Octave:abbreviated-property-match");
   m_error_system.disable_warning ("Octave:colon-nonscalar-argument");
@@ -2066,7 +2204,8 @@ void interpreter::maximum_braindamage (void)
   m_error_system.disable_warning ("Octave:possible-matlab-short-circuit-operator");
 }
 
-void interpreter::execute_pkg_add (const std::string& dir)
+void
+interpreter::execute_pkg_add (const std::string& dir)
 {
   try
     {
@@ -2082,4 +2221,34 @@ void interpreter::execute_pkg_add (const std::string& dir)
     }
 }
 
+// Execute commands from a file and catch potential exceptions in a consistent
+// way.  This function should be called anywhere we might parse and execute
+// commands from a file before we have entered the main loop in
+// toplev.cc.
+
+int
+interpreter::safe_source_file (const std::string& file_name,
+                               const std::string& context,
+                               bool verbose, bool require_file)
+{
+  try
+    {
+      source_file (file_name, context, verbose, require_file);
+    }
+  catch (const interrupt_exception&)
+    {
+      recover_from_exception ();
+
+      return 1;
+    }
+  catch (const execution_exception& ee)
+    {
+      handle_exception (ee);
+
+      return 1;
+    }
+
+  return 0;
+}
+
 OCTAVE_END_NAMESPACE(octave)
